VERSION 5.00
Begin VB.UserControl PopMenu 
   ClientHeight    =   1005
   ClientLeft      =   0
   ClientTop       =   0
   ClientWidth     =   1950
   InvisibleAtRuntime=   -1  'True
   ScaleHeight     =   1005
   ScaleWidth      =   1950
   ToolboxBitmap   =   "cspPMenu.ctx":0000
End
Attribute VB_Name = "PopMenu"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

' The sub classing control.  We need to use this because it
' ensures consistent sub-classing even if the form is being
' sub-classed by another control:
Implements ISubclass

' Need this to extract DRAWITEM and MEASUREITEM information
' from the owner draw messages:
Private Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" ( _
    lpvDest As Any, lpvSource As Any, ByVal cbCopy As Long)
' The messages we will intercept:
Private Const WM_MENUSELECT = &H11F
Private Const WM_MEASUREITEM = &H2C
Private Const WM_DRAWITEM = &H2B
Private Const WM_COMMAND = &H111
Private Const WM_MENUCHAR = &H120
Private Const WM_SYSCOMMAND = &H112
Private Const WM_INITMENUPOPUP = &H117
Private Const WM_WININICHANGE = &H1A

' Menu information:
Private Type tMenuItem
    sHelptext As String
    sCaption As String
    sAccelerator As String
    sShortCutDisplay As String
    sKey As String
    iShortCutShiftMask As Integer
    iShortCutShiftKey As Integer
    lId As Long
    lActualID As Long       ' The ID gets modified if we add a sub-menu to the hMenu of the popup
    lItemData As Long
    lIndex As Long
    lParentId As Long
    lIconIndex As Long
    bChecked As Boolean
    bEnabled As Boolean
    hMenu As Long
    lWidth As Long
    bCreated As Boolean
    bIsAVBMenu As Boolean
    lShortCutStartPos As Long
    bMarkTODestroy As Boolean
End Type

Private Type tVBMenuInfo
    sCaption As String
    sName As String
    bHasIndex As Boolean
    iIndex As Long
    bUsed As Boolean
End Type

' Array of menu items
Private m_tMI() As tMenuItem
Private m_iMenuCount As Long
' Next id to choose for a menu item:
Private m_lLastMaxId As Long
' Height of a menu item:
Private m_lMenuItemHeight As Long

' Hwnd of parent form:
Private m_hWndParent As Long

' Handle to image list for drawing icons:
Private m_hIml As Long
' Where to get a tick icon for checked stuff:
Private m_lTickIconIndex As Long

' Sub menus we have created:
Private m_hSubMenus() As Long
Private m_lSubMenuCount As Long

' Subclassing response
Private m_emr As EMsgResponse

' When adding system menu items, set their id to this:
Private m_lNextSysMenuID As Long
Private Const WM_MENUBASE = &H2000&

' Display stuff, used to draw the control and also
' to evaluate menu font item sizes:
Private m_HDC As Long
Private m_hBMPOLd As Long
Private m_hBmp As Long
Private m_hFntOld As Long
Private m_cNCM As cNCMetrics
Private m_bGotFont As Boolean

' Events:
Public Event Click(ItemNumber As Long)
Attribute Click.VB_Description = "Raised when a menu item is clicked."
Public Event SystemMenuClick(ItemNumber As Long)
Attribute SystemMenuClick.VB_Description = "Raised when a system menu item is clicked."
Public Event ItemHighlight(ItemNumber As Long, bEnabled As Boolean, bSeparator As Boolean)
Attribute ItemHighlight.VB_Description = "Raised when a  menu item is highlighted."
Public Event SystemMenuItemHighlight(ItemNumber As Long, bEnabled As Boolean, bSeparator As Boolean)
Attribute SystemMenuItemHighlight.VB_Description = "Raised when a system menu item is highlighted."
Public Event MenuExit()
Attribute MenuExit.VB_Description = "Raised when a menu is exited."
Public Event InitPopupMenu(ParentItemNumber As Long)
Attribute InitPopupMenu.VB_Description = "Raised when a popup menu is about to be shown. You can change the contents of the popup menu in this event before the menu is shown."
Public Event WinIniChange()
Attribute WinIniChange.VB_Description = "Raised when there is a change to the display settings for the system."

' Enumerations:
Public Enum CSPPMENUSysCommandConstants

     SC_RESTORE = &HF120&
     SC_MOVE = &HF010&
     SC_SIZE = &HF000&
     SC_MAXIMIZE = &HF030&
     SC_MINIMIZE = &HF020&
     SC_CLOSE = &HF060&
     
     SC_ARRANGE = &HF110&
     SC_HOTKEY = &HF150&
     SC_HSCROLL = &HF080&
     SC_KEYMENU = &HF100&
     SC_MANAGER_CONNECT = &H1&
     SC_MANAGER_CREATE_SERVICE = &H2&
     SC_MANAGER_ENUMERATE_SERVICE = &H4&
     SC_MANAGER_LOCK = &H8&
     SC_MANAGER_MODIFY_BOOT_CONFIG = &H20&
     SC_MANAGER_QUERY_LOCK_STATUS = &H10&
     SC_MOUSEMENU = &HF090&
     SC_NEXTWINDOW = &HF040&
     SC_PREVWINDOW = &HF050&
     SC_SCREENSAVE = &HF140&
     SC_TASKLIST = &HF130&
     SC_VSCROLL = &HF070&
     SC_ZOOM = SC_MAXIMIZE
     SC_ICON = SC_MINIMIZE
     
End Enum

Public Property Get MenuIndex(ByVal vKey As Variant) As Long
Attribute MenuIndex.VB_Description = "Returns the internal index of a menu item given its Key."
Dim i As Long
    MenuIndex = -1
    If (IsNumeric(vKey)) Then
        i = CLng(vKey)
        If (i > 0) And (i <= m_iMenuCount) Then
            MenuIndex = i
        Else
            Err.Raise 9, App.EXEName & ".cPopMenu"
        End If
    Else
        For i = 1 To m_iMenuCount
            If (m_tMI(i).sKey = vKey) Then
                MenuIndex = i
                Exit Property
            End If
        Next i
        Err.Raise 9, App.EXEName & ".cPopMenu"
    End If
End Property
Public Property Get MenuKey(ByVal lIndex As Long) As String
Attribute MenuKey.VB_Description = "Returns the key for a given menu item given the index."
    MenuKey = m_tMI(lIndex).sKey
End Property
Public Property Let MenuKey(ByVal lIndex As Long, ByVal sKey As String)
    If (pbIsValidKey(sKey)) Then
        m_tMI(lIndex).sKey = sKey
    End If
End Property
Private Function pbIsValidKey(ByRef sKey As String) As Boolean
Dim i As Long
Dim bInvalid As Boolean
    If (Trim$(sKey) = "") Then
        sKey = Trim$(sKey)
        ' you're allowed to have a null key:
        pbIsValidKey = True
    Else
        For i = 1 To m_iMenuCount
            If (m_tMI(i).sKey = sKey) Then
                bInvalid = True
                Exit For
            End If
        Next i
        If (bInvalid) Then
            Err.Raise 457, App.EXEName & ".cPopMenu"
        Else
            pbIsValidKey = True
        End If
    End If
End Function
Public Property Let TickIconIndex( _
        ByVal lIconIndex As Long _
    )
Attribute TickIconIndex.VB_Description = "Gets/sets  zero based icon index of an icon in an ImageList to associate with checked menu items without their own icon."
    m_lTickIconIndex = lIconIndex
End Property
Public Property Get TickIconIndex() As Long
    TickIconIndex = m_lTickIconIndex
End Property
Public Property Get SystemMenuCaption( _
        ByVal lPosition As Long _
    ) As String
Attribute SystemMenuCaption.VB_Description = "Sets the caption for a given system menu item."
Dim tMII As MENUITEMINFO
Dim hSysMenu As Long
    hSysMenu = GetSystemMenu(m_hWndParent, 0)
    If (hSysMenu <> 0) Then
        tMII.fMask = MIIM_DATA
        tMII.cch = 127
        tMII.dwTypeData = String$(128, 0)
        tMII.cbSize = LenB(tMII)
        GetMenuItemInfo hSysMenu, (lPosition - 1), 1, tMII
        SystemMenuCaption = left$(tMII.dwTypeData, tMII.cch)
    End If
End Property
Public Property Get SystemMenuCount() As Long
Attribute SystemMenuCount.VB_Description = "Returns the number of items in the container's system menu."
Dim hSysMenu As Long
    hSysMenu = GetSystemMenu(m_hWndParent, 0)
    If (hSysMenu <> 0) Then
        SystemMenuCount = GetMenuItemCount(hSysMenu)
    End If
End Property
Public Sub SystemMenuRemoveItem( _
        ByVal lPosition As Long _
    )
Attribute SystemMenuRemoveItem.VB_Description = "Removes an item from the container's system menu."
Dim hSysMenu As Long
    hSysMenu = GetSystemMenu(m_hWndParent, 0)
    If (hSysMenu <> 0) Then
        RemoveMenu hSysMenu, (lPosition - 1), MF_BYPOSITION
    End If
End Sub
Public Function SystemMenuAppendItem( _
        ByVal sCaption As String _
    ) As Long
Attribute SystemMenuAppendItem.VB_Description = "Adds an item to the form's system menu."
Dim hSysMenu As Long
    hSysMenu = GetSystemMenu(m_hWndParent, 0)
    If (hSysMenu <> 0) Then
        If (sCaption = "-") Then
            AppendMenuBylong hSysMenu, MF_SEPARATOR, m_lNextSysMenuID, m_lNextSysMenuID
        Else
            AppendMenuByString hSysMenu, MF_STRING, m_lNextSysMenuID, sCaption
        End If
        SystemMenuAppendItem = m_lNextSysMenuID
        m_lNextSysMenuID = m_lNextSysMenuID + 1
    End If
End Function
Public Sub SystemMenuRestore()
Attribute SystemMenuRestore.VB_Description = "Restores the container's system menu to its original state."
    GetSystemMenu m_hWndParent, 1
    m_lNextSysMenuID = WM_MENUBASE
End Sub
Private Function plParseMenuChar( _
        ByVal hMenu As Long, _
        ByVal iChar As Integer _
    ) As Long
Dim sChar As String
Dim lPos As Long
Dim l As Long
    
    sChar = UCase$(Chr$(iChar))
    For l = 1 To m_iMenuCount
        If (m_tMI(l).hMenu = hMenu) Then
            If (m_tMI(l).sAccelerator = sChar) Then
                plParseMenuChar = &H20000 + lPos
                ' Debug.Print "Found Menu Char"
                Exit Function
            End If
            lPos = lPos + 1
        End If
    Next l
    
End Function

Public Property Let ImageList( _
        ByRef vImageList As Variant _
    )
Attribute ImageList.VB_Description = "Associates an ImageList with the control for drawing icons.  Either pass a VB image list or a valid API hImageList handle to this property."
    If (VarType(vImageList) = vbLong) Then
        ' Assume a handle to an image list:
        m_hIml = vImageList
    ElseIf (VarType(vImageList) = vbObject) Then
        ' Assume a VB image list:
        On Error Resume Next
        ' Get the image list initialised..
        vImageList.ListImages(1).Draw 0, 0, 0, 1
        m_hIml = vImageList.hImageList
        If (Err.Number = 0) Then
            ' OK
        Else
            Debug.Print "Failed to Get Image list Handle", "cVGrid.ImageList"
        End If
        On Error GoTo 0
    End If
End Property
Public Property Get Caption( _
        ByVal vKey As Variant _
    ) As String
Attribute Caption.VB_Description = "Gets/sets the caption for a menu item.  Same as VB menu caption property, except it also works for new menus added to the control."
Dim lIndex As Long
    lIndex = MenuIndex(vKey)
    If (lIndex > 0) Then
        Caption = m_tMI(lIndex).sCaption
    End If
End Property
Public Property Let Caption( _
        ByVal vKey As Variant, _
        ByVal sCaption As String _
    )
Dim lIndex As Long
    lIndex = MenuIndex(vKey)
    If (lIndex > 0) Then
        m_tMI(lIndex).sCaption = sCaption
        m_tMI(lIndex).sAccelerator = psExtractAccelerator(sCaption)
        If (pbIsTopLevelmenu(lIndex)) Then
            ' Somehow we need to re-measure all the top menu items.
            ' How do we do this? Please tell me if you know!
            ' Do we call InvalidateRect on the menu area?
        End If
    End If
End Property
Public Property Get Enabled( _
        ByVal vKey As Variant _
    ) As Boolean
Attribute Enabled.VB_Description = "Gets/sets the enabled state for a menu item.  Same as VB menu enabled property, except it also works for new menus added to the control."
Dim tMII As MENUITEMINFO
Dim lIndex As Long
    lIndex = MenuIndex(vKey)
    If (lIndex > 0) Then
        tMII.fMask = MIIM_STATE
        tMII.cbSize = LenB(tMII)
        GetMenuItemInfo m_tMI(lIndex).hMenu, m_tMI(lIndex).lActualID, False, tMII
        m_tMI(lIndex).bEnabled = Not ((tMII.fState And MFS_DISABLED) = MFS_DISABLED)
        Enabled = m_tMI(lIndex).bEnabled
    End If
End Property
Public Property Let Enabled( _
        ByVal vKey As Variant, _
        ByVal bEnabled As Boolean _
    )
Dim lFlag As Long
Dim lFlagNot As Long
Dim lIndex As Long
    lIndex = MenuIndex(vKey)
    If (lIndex > 0) Then
        m_tMI(lIndex).bEnabled = bEnabled
        If (bEnabled) Then
            lFlag = MF_ENABLED
            lFlagNot = MF_GRAYED
        Else
            lFlag = MF_DISABLED
            lFlagNot = MF_GRAYED
        End If
        pSetMenuFlag lIndex, lFlag, lFlagNot
    End If
End Property
Public Property Get Checked( _
        ByVal vKey As Variant _
    ) As Boolean
Attribute Checked.VB_Description = "Gets/sets the checked property for a menu item.  Same as VB menu caption property, except it also works for new menus added to the control."
Dim tMII As MENUITEMINFO
Dim lIndex As Long
    lIndex = MenuIndex(vKey)
    If (lIndex > 0) Then
        tMII.fMask = MIIM_STATE
        tMII.cbSize = LenB(tMII)
        GetMenuItemInfo m_tMI(lIndex).hMenu, m_tMI(lIndex).lActualID, False, tMII
        m_tMI(lIndex).bChecked = ((tMII.fState And MFS_CHECKED) = MFS_CHECKED)
        Checked = m_tMI(lIndex).bChecked
    End If
End Property
Public Property Let Checked( _
        ByVal vKey As Variant, _
        ByVal bChecked As Boolean _
    )
Dim lFlag As Long
Dim lFlagNot As Long
Dim lIndex As Long
    lIndex = MenuIndex(vKey)
    If (lIndex > 0) Then
        m_tMI(lIndex).bChecked = bChecked
        If (bChecked) Then
            lFlag = MF_CHECKED
            lFlagNot = 0
        Else
            lFlag = 0
            lFlagNot = MF_CHECKED
        End If
        pSetMenuFlag lIndex, lFlag, lFlagNot
    End If
End Property
Public Property Get HelpText( _
        ByVal vKey As Variant _
    ) As String
Attribute HelpText.VB_Description = "Gets/sets a help string to be associated with a menu item."
Dim lIndex As Long
    lIndex = MenuIndex(vKey)
    If (lIndex > 0) Then
        HelpText = m_tMI(lIndex).sHelptext
    End If
End Property
Public Property Let HelpText( _
        ByVal vKey As Variant, _
        ByVal sHelptext As String _
    )
Dim lIndex As Long
    lIndex = MenuIndex(vKey)
    If (lIndex > 0) Then
        m_tMI(lIndex).sHelptext = sHelptext
    End If
End Property
Public Property Let ItemIcon( _
        ByVal vKey As Variant, _
        ByVal lIconIndex As Long _
    )
Attribute ItemIcon.VB_Description = "Gets/sets the zero based icon index of an icon in an ImageList to associate with a menu item."
Dim lPrevIconIndex As Long
Dim lIndex As Long
    lIndex = MenuIndex(vKey)
    If (lIndex > 0) Then
        lPrevIconIndex = m_tMI(lIndex).lIconIndex
        m_tMI(lIndex).lIconIndex = lIconIndex
        If (((lPrevIconIndex = -1) Or (lIconIndex = -1)) And (lPrevIconIndex <> lIconIndex)) Then
            If (pbIsTopLevelmenu(lIndex)) Then
                ' Somehow we need to re-measure all the top menu items.
                ' How do we do this?
            End If
        End If
    End If
End Property
Public Property Get ItemIcon( _
        ByVal vKey As Variant _
    ) As Long
Dim lIndex As Long
    lIndex = MenuIndex(vKey)
    If (lIndex > 0) Then
        ItemIcon = m_tMI(lIndex).lIconIndex
    End If
End Property
Public Property Get ItemData( _
        ByVal vKey As Variant _
    ) As Long
Attribute ItemData.VB_Description = "Gets/sets a long variable to associate with a menu item."
Dim lIndex As Long
    lIndex = MenuIndex(vKey)
    If (lIndex > 0) Then
        ItemData = m_tMI(lIndex).lItemData
    End If
End Property
Public Property Let ItemData( _
        ByVal vKey As Variant, _
        ByVal lItemData As Long _
    )
Dim lIndex As Long
    lIndex = MenuIndex(vKey)
    If (lIndex > 0) Then
        m_tMI(lIndex).lItemData = lItemData
    End If
End Property
Public Property Get hPopupMenu( _
        ByVal vKey As Variant _
    ) As Long
Attribute hPopupMenu.VB_Description = "Returns an API handle to the popup menu underneath a given menu, or 0 if there is no submenu."
Dim tMII As MENUITEMINFO
Dim lIndex As Long
    lIndex = MenuIndex(vKey)
    If (lIndex > 0) Then
        tMII.fMask = MIIM_SUBMENU
        tMII.cbSize = LenB(tMII)
        GetMenuItemInfo m_tMI(lIndex).hMenu, m_tMI(lIndex).lActualID, False, tMII
        hPopupMenu = tMII.hSubMenu
    End If
End Property
Public Property Get PositionInMenu( _
        ByVal vKey As Variant _
    ) As Long
Attribute PositionInMenu.VB_Description = "Returns the position within a menu for the given menu index."
Dim lIndex As Long
    lIndex = MenuIndex(vKey)
    If (lIndex > 0) Then
    
    End If
End Property
Public Property Get NextSibling( _
        ByVal vKey As Variant _
    ) As Long
Attribute NextSibling.VB_Description = "Returns the next menu along from the given menu item, ignoring popup menus."
Dim lIndex As Long
Dim lParentId As Long
Dim l As Long
    lIndex = MenuIndex(vKey)
    If (lIndex > 0) Then
        ' The next sibling is the next item with the same parent id:
        lParentId = m_tMI(lIndex).lParentId
        For l = lIndex + 1 To m_iMenuCount
            If (m_tMI(l).lParentId = lParentId) Then
                NextSibling = l
                Exit For
            End If
        Next l
    End If
End Property
Public Property Get SiblingCount( _
        ByVal vKey As Variant _
    ) As Long
Attribute SiblingCount.VB_Description = "Returns the number of items in the menu containing the menu with the specified index."
Dim lIndex As Long
Dim lParentId As Long
Dim l As Long
Dim iCount As Long
    lIndex = MenuIndex(vKey)
    If (lIndex > 0) Then
        ' The sibling count is the total of items with the same parent id:
        lParentId = m_tMI(lIndex).lParentId
        For l = 1 To m_iMenuCount
            If (m_tMI(l).lParentId = lParentId) Then
                iCount = iCount + 1
            End If
        Next l
        SiblingCount = iCount
    End If
        
End Property
Public Property Get HasChildren( _
        ByVal vKey As Variant _
    )
Attribute HasChildren.VB_Description = "Returns whether a given menu item has a child menu."
Dim lIndex As Long
    lIndex = MenuIndex(vKey)
    If (lIndex > 0) Then
        ' An item has children if there are any menu items with the
        ' the parent id set to the index of this item:
        HasChildren = (FirstChild(lIndex) <> 0)
    End If
End Property
Public Property Get FirstChild( _
        ByVal vKey As Variant _
    ) As Long
Attribute FirstChild.VB_Description = "Returns the index of the first child of a menu item, or 0 if there are no children."
Dim lIndex As Long
Dim lId As Long
Dim l As Long
    lIndex = MenuIndex(vKey)
    If (lIndex > 0) Then
        ' Return the first item which has a ParentId = lIndex:
        lId = m_tMI(lIndex).lId
        For l = 1 To m_iMenuCount
            If (m_tMI(l).lParentId = lId) Then
                FirstChild = l
                Exit For
            End If
        Next l
    End If
End Property
Public Property Get Parent( _
        ByVal vKey As Variant _
    ) As Long
Attribute Parent.VB_Description = "Returns the parent index of a menu, or zero if there isn't one."
Dim lIndex As Long
    lIndex = MenuIndex(vKey)
    If (lIndex > 0) Then
        Parent = plGetIndexForId(m_tMI(lIndex).lParentId)
    End If
End Property
Public Property Get UltimateParent( _
        ByVal vKey As Variant _
    ) As Long
Attribute UltimateParent.VB_Description = "Returns the index of the top level parent item for a given menu item."
Dim lIndex As Long
Dim lR As Long
    lIndex = MenuIndex(vKey)
    If (lIndex > 0) Then
        lR = plGetIndexForId(m_tMI(lIndex).lParentId)
        If (lR <> 0) Then
            Do While m_tMI(lR).lParentId <> 0
                lR = plGetIndexForId(m_tMI(lR).lParentId)
            Loop
        End If
        If (lR = 0) Then
            UltimateParent = lIndex
        Else
            UltimateParent = lR
        End If
    End If
End Property
Private Sub pSetMenuFlag( _
        ByVal lIndex As Long, _
        ByVal lFlag As Long, _
        ByVal lFlagNot As Long _
    )
Dim tMII As MENUITEMINFO
Dim lFlags As Long

    lFlags = plMenuFlags(lIndex)
    lFlags = (lFlags Or MF_OWNERDRAW) And Not MF_STRING
    tMII.fMask = MIIM_SUBMENU
    tMII.cbSize = LenB(tMII)
    GetMenuItemInfo m_tMI(lIndex).hMenu, m_tMI(lIndex).lActualID, False, tMII
    If (tMII.hSubMenu <> 0) Then
        lFlags = lFlags Or MF_POPUP
    End If
    lFlags = lFlags And Not MF_BYPOSITION Or MF_BYCOMMAND
    
    lFlags = lFlags Or lFlag
    lFlags = lFlags And Not lFlagNot
    
    ModifyMenuByLong m_tMI(lIndex).hMenu, m_tMI(lIndex).lActualID, lFlags, m_tMI(lIndex).lActualID, m_tMI(lIndex).lActualID
    
End Sub
Property Get IndexForMenuHierarchyParamArray( _
        ParamArray vHierarchy() As Variant _
    ) As Long
Attribute IndexForMenuHierarchyParamArray.VB_Description = "Gets the index for a menu given the hierarchy position as a ParamArray.  An example hierarchy would be 2,5 for the Edit-Copy menu in Notepad."
Dim lH() As Long
Dim l As Long
    ReDim lH(LBound(vHierarchy) To UBound(vHierarchy)) As Long
    For l = LBound(vHierarchy) To UBound(vHierarchy)
        lH(l) = vHierarchy(l)
    Next l
    IndexForMenuHierarchyParamArray = IndexForMenuHierarchy(lH())
End Property
Property Get IndexForMenuHierarchy( _
        ByRef lHierarchy() As Long _
    ) As Long
Attribute IndexForMenuHierarchy.VB_Description = "Gets the index for a menu given the hierarchy position as an array of longs.  An example hierarchy would be 2,5 for the Edit-Copy menu in Notepad."
Dim l As Long
Dim lEnd As Long
Dim hMenuSeek As Long
Dim lRet As Long
Dim lFindIndex As Long

    hMenuSeek = GetMenu(m_hWndParent)
    lEnd = UBound(lHierarchy, 1)
    For l = LBound(lHierarchy, 1) To lEnd
        lFindIndex = plFindItemInMenu(hMenuSeek, lHierarchy(l))
        If (lFindIndex <> 0) Then
            If (l = lEnd) Then
                lRet = lFindIndex
            Else
                hMenuSeek = hPopupMenu(lFindIndex)
                If (hMenuSeek = 0) Then
                    Exit For
                End If
            End If
        Else
            Exit For
        End If
    Next l
    IndexForMenuHierarchy = lRet
End Property
Public Sub GetHierarchyForIndexPosition( _
        ByVal vKey As Variant, _
        ByRef lHierarchy() As Long _
    )
Attribute GetHierarchyForIndexPosition.VB_Description = "Returns a hierarchy for a menu in an array of longs.  An example hierarchy would be 2,5 for the Edit-Copy menu in Notepad."
Dim sItems() As String
Dim lIndex As Long
    lIndex = MenuIndex(vKey)
    If (lIndex > 0) Then
        pHierarchyForIndex lIndex, lHierarchy(), sItems()
    End If
End Sub
Property Get HierarchyPath( _
        ByVal vKey As Variant, _
        ByVal lStartLevel As Long, _
        ByVal sSeparator As String _
    ) As String
Attribute HierarchyPath.VB_Description = "Gets the text of a menu tree as a path.  For example, Favourites-Steves-vbaccelerator would return ""Favourites\\Steves\\vbaccelerator"""
Dim sItems() As String
Dim lH() As Long
Dim lItem As Long
Dim sOut As String
Dim lIndex As Long
    lIndex = MenuIndex(vKey)
    If (lIndex > 0) Then
        pHierarchyForIndex lIndex, lH(), sItems()
        For lItem = lStartLevel To UBound(sItems)
            sOut = sOut & sItems(lItem) & sSeparator
        Next lItem
        If Len(sOut) > 0 Then
            sOut = left$(sOut, Len(sOut) - 1)
            HierarchyPath = sOut
        End If
    End If
End Property
Private Function pHierarchyForIndex( _
        ByVal lIndex As Long, _
        ByRef lHierarchy() As Long, _
        ByRef sItems() As String _
    ) As String
Dim lH() As Long
Dim sI() As String
Dim lItems As Long
Dim hMenuSeek As Long
Dim lPid As Long
Dim bComplete As Boolean
Dim l As Long
Dim lNewIndex As Long
Dim sOut As String

    Erase lHierarchy
    Erase sItems
    ' Now determine the hierarchy for this item:
    hMenuSeek = m_tMI(lIndex).hMenu
    Do
        lItems = lItems + 1
        ReDim Preserve lH(1 To lItems) As Long
        ReDim Preserve sI(1 To lItems) As String
        lH(lItems) = plMenuPositionFOrIndex(hMenuSeek, lIndex)
        sI(lItems) = m_tMI(lIndex).sCaption
        lPid = m_tMI(lIndex).lParentId
        If (lPid <> 0) Then
            lNewIndex = plGetIndexForId(m_tMI(lIndex).lParentId)
            ' Debug.Print lNewIndex
            lIndex = lNewIndex
            hMenuSeek = m_tMI(lIndex).hMenu
        Else
            bComplete = True
        End If
    Loop While Not (bComplete)
    
    ReDim lHierarchy(1 To lItems) As Long
    ReDim sItems(1 To lItems) As String
    For l = lItems To 1 Step -1
        lHierarchy(l) = lH(lItems - l + 1)
        sItems(l) = sI(lItems - l + 1)
    Next l

End Function

Private Function IndexForId( _
        ByVal lId As Long _
    )
Dim lItem As Long
    For lItem = 1 To m_iMenuCount
        If (m_tMI(lItem).lActualID = lId) Then
            IndexForId = lItem
            Exit For
        End If
    Next lItem
End Function
Private Function plMenuPositionFOrIndex( _
        ByVal hMenuSeek As Long, _
        ByVal lIndex As Long _
    ) As Long
Dim l As Long
Dim lPos As Long
    For l = 1 To lIndex
        If (m_tMI(l).hMenu = hMenuSeek) Then
            lPos = lPos + 1
        End If
    Next l
    plMenuPositionFOrIndex = lPos
End Function
Private Function plFindItemInMenu( _
        ByVal hMenuSeek As Long, _
        ByVal lPosition As Long _
    ) As Long
Dim lPos As Long
Dim l As Long
    For l = 1 To m_iMenuCount
        If (m_tMI(l).hMenu = hMenuSeek) Then
            lPos = lPos + 1
            If (lPos = lPosition) Then
                plFindItemInMenu = l
                Exit For
            End If
        End If
    Next l
End Function
Public Function ClearSubMenusOfItem( _
        ByVal vKey As Variant _
    ) As Long
Attribute ClearSubMenusOfItem.VB_Description = "Removes all sub-menus from a given menu."
Dim hMenu As Long
Dim iMenu As Long
Dim lIndex As Long
    lIndex = MenuIndex(vKey)
    If (lIndex > 0) Then
        ' The idea is to leave just the submenu
        ' but with nothing in it:
        
        ' The ActualID of a sub-menu will be the
        ' handle to the submenu:
        hMenu = m_tMI(lIndex).lActualID
            
        ' Now remove all the items in the sub-menu,
        ' mark them for destruction and also do
        ' any sub-menus they may have:
        For iMenu = m_iMenuCount To 1 Step -1
            If (iMenu <= m_iMenuCount) Then
                If (m_tMI(iMenu).hMenu = hMenu) Then
                    pRemoveItem iMenu
                End If
            End If
        Next iMenu
        
        For iMenu = 1 To m_iMenuCount
            If (m_tMI(iMenu).lActualID = hMenu) Then
                ClearSubMenusOfItem = iMenu
                Exit For
            End If
        Next iMenu
    End If
End Function
Public Sub RemoveItem( _
        ByVal vKey As Variant _
    )
Attribute RemoveItem.VB_Description = "Removes an Item from a menu."
Dim lIndex As Long
    lIndex = MenuIndex(vKey)
    If (lIndex > 0) Then
        pRemoveItem lIndex
    End If
End Sub
Private Sub pRemoveItem( _
        ByVal lIndex As Long _
    )
Dim hMenusToDestroy() As Long
Dim lCount As Long
Dim lDestroy As Long
Dim lRealCount As Long
Dim lR As Long
Dim lMaxID As Long

    ' Remove the Item:
    lR = RemoveMenu(m_tMI(lIndex).hMenu, m_tMI(lIndex).lActualID, MF_BYCOMMAND)
    m_tMI(lIndex).bMarkTODestroy = True
    ' Loop though all the children of the item at Index and determine
    ' what there is to remove:
    pRemoveSubMenus m_tMI(lIndex).lActualID, 1, hMenusToDestroy(), lCount
            
    ' Destroy the menus:
    For lDestroy = 1 To lCount
        DestroyMenu hMenusToDestroy(lDestroy)
        Debug.Print "Destroyed sub-menu:" & hMenusToDestroy(lDestroy)
    Next lDestroy
    
    ' Now repopulate the array & sort out the indexes to remove
    ' the indexes marked for deletion:
    If (lCount > 0) Or (lR <> 0) Then
        lRealCount = 0
        For lIndex = 1 To m_iMenuCount
            If Not (m_tMI(lIndex).bMarkTODestroy) Then
                If (GetMenuItemCount(m_tMI(lIndex).lActualID) = -1) Then
                    If (m_tMI(lIndex).lActualID > lMaxID) Then
                        lMaxID = m_tMI(lIndex).lActualID
                    End If
                End If
                lRealCount = lRealCount + 1
                If (lRealCount <> lIndex) Then
                    ' This would have been nice but it is a no
                    ' because of those dammed bstrs:
                        ' CopyMemory m_tMI(lRealCOunt), m_tMI(lIndex), LenB(m_tMI(lIndex))
                    With m_tMI(lRealCount)
                        .bChecked = m_tMI(lIndex).bChecked
                        .bCreated = m_tMI(lIndex).bCreated
                        .bEnabled = m_tMI(lIndex).bEnabled
                        .bIsAVBMenu = m_tMI(lIndex).bIsAVBMenu
                        .bMarkTODestroy = m_tMI(lIndex).bMarkTODestroy
                        .hMenu = m_tMI(lIndex).hMenu
                        .iShortCutShiftKey = m_tMI(lIndex).iShortCutShiftKey
                        .iShortCutShiftMask = m_tMI(lIndex).iShortCutShiftMask
                        .lActualID = m_tMI(lIndex).lActualID
                        .lIconIndex = m_tMI(lIndex).lIconIndex
                        .lId = m_tMI(lIndex).lId
                        .lIndex = m_tMI(lIndex).lIndex
                        .lItemData = m_tMI(lIndex).lItemData
                        .lParentId = m_tMI(lIndex).lParentId
                        .lShortCutStartPos = m_tMI(lIndex).lShortCutStartPos
                        .lWidth = m_tMI(lIndex).lWidth
                        .sAccelerator = m_tMI(lIndex).sAccelerator
                        .sCaption = m_tMI(lIndex).sCaption
                        .sHelptext = m_tMI(lIndex).sHelptext
                        .sShortCutDisplay = m_tMI(lIndex).sShortCutDisplay
                        ' There. I'd have never have bothered if it wasn't for
                        ' VB5's click dot and it shows you the options!
                    End With
                End If
            End If
        Next lIndex
        ReDim Preserve m_tMI(1 To lRealCount) As tMenuItem
        m_iMenuCount = lRealCount
        If (lMaxID > m_iMenuCount) Then
            m_lLastMaxId = lMaxID
        Else
            m_lLastMaxId = m_iMenuCount
        End If
    End If
    'Debug.Print m_iMenuCount, m_lLastMaxId
    
End Sub
Private Sub pRemoveSubMenus( _
        ByVal lParentId As Long, _
        ByVal lStartIndex As Long, _
        ByRef hMenusToDestroy() As Long, _
        ByRef lMenuToDestroyCount As Long _
    )
Dim lIndex As Long
    
    For lIndex = 1 To m_iMenuCount
        If (m_tMI(lIndex).lParentId = lParentId) Then
            m_tMI(lIndex).bMarkTODestroy = True
            pAddToDestroyArray m_tMI(lIndex).hMenu, hMenusToDestroy(), lMenuToDestroyCount
            pRemoveSubMenus m_tMI(lIndex).lActualID, lIndex, hMenusToDestroy(), lMenuToDestroyCount
        End If
    Next lIndex
End Sub
Private Sub pAddToDestroyArray( _
        ByVal hMenu As Long, _
        ByRef hMenusToDestroy() As Long, _
        ByRef lMenuToDestroyCount As Long _
    )
Dim lIndex As Long
Dim bFound As Boolean
    For lIndex = 1 To lMenuToDestroyCount
        If (hMenusToDestroy(lIndex) = hMenu) Then
            bFound = True
            Exit For
        End If
    Next lIndex
    If Not (bFound) Then
        lMenuToDestroyCount = lMenuToDestroyCount + 1
        ReDim Preserve hMenusToDestroy(1 To lMenuToDestroyCount) As Long
        hMenusToDestroy(lMenuToDestroyCount) = hMenu
    End If
End Sub
Private Sub RaiseMenuExitEvent()
    RaiseEvent MenuExit
End Sub
Private Function RaiseClickEvent(lId As Long) As Boolean
' Return true from this if we have completely handled the
' click on our own:
Dim lIndex As Long

   ' Check whether this is an MDI child system menu command:
   If (lId = SC_SIZE) Or (lId = SC_MOVE) Or (lId = SC_MINIMIZE) Or (lId = SC_MAXIMIZE) Or (lId = SC_RESTORE) Or (lId = SC_CLOSE) Then
      RaiseClickEvent = False
   Else

      ' Find the Index of this menu id within our own array:
      lIndex = plGetIndexForId(lId)
      
      ' If we find it, then raise a click event for it:
      If (lIndex > 0) Then
      
          ' Send a click event with the index:
          RaiseEvent Click(lIndex)
          
          ' If this was one of the VB menu entries we have
          ' subclassed, we want to return false.  Then the
          ' click will filter through to the original Click
          ' event so your code should work as normal:
          If Not (m_tMI(lIndex).bIsAVBMenu) Then
              RaiseClickEvent = True
          End If
          
      Else
          ' This is a problem.  We've got a click on
          ' a menu id which doesn't seem to be any
          ' of the menu items of the form.  It shouldn't
          ' happen, but return true anyway so we eat
          ' the message.  This should prevent unwanted
          ' interference with any other controls on the
          ' form which seem to think the message is their
          ' own...
          Debug.Print "Failed to find index"
          RaiseClickEvent = True
      End If
   End If
End Function
Private Function pbIdIsSysMenuId( _
        ByVal lId As Long _
    ) As Boolean
' Determine whether the menu id lId is actually one of the standard
' system menu items, or if it is an item which has been added to the
' system menu by this control:

    Select Case lId
    Case SC_RESTORE, SC_MOVE, SC_SIZE, SC_MAXIMIZE, SC_MINIMIZE, SC_CLOSE
        ' This is a standard menu id item:
        pbIdIsSysMenuId = True
    Case WM_MENUBASE To m_lNextSysMenuID - 1
        ' This is a new item added to the system menu by this control:
        pbIdIsSysMenuId = True
    End Select
    
End Function
Private Sub RaiseHighlightEvent(lId As Long)
Dim lIndex As Long

    ' Debug.Print lItem
    lIndex = plGetIndexForId(lId)
    If (lIndex > 0) Then
        RaiseEvent ItemHighlight(lIndex, m_tMI(lIndex).bEnabled, (Trim$(m_tMI(lIndex).sCaption = "-")))
    Else
        ' It may be a sys menu item:
        If (pbIdIsSysMenuId(lId)) Then
            Dim tMII As MENUITEMINFO
            Dim hSysMenu As Long
            hSysMenu = GetSystemMenu(m_hWndParent, 0)
            If (hSysMenu <> 0) Then
                tMII.fMask = MIIM_STATE
                tMII.cbSize = LenB(tMII)
                GetMenuItemInfo hSysMenu, lId, False, tMII
            End If
            RaiseEvent SystemMenuItemHighlight(lId, ((tMII.fState And MFS_DISABLED) <> MFS_DISABLED), False)
        Else
            Debug.Print "Failed to find Index for Highlight Id:", lId, lIndex
        End If
    End If
End Sub
Private Sub DrawItem( _
        ByRef lParam As Long, _
        ByRef wParam As Long _
    )
Dim tDI As DRAWITEMSTRUCT
Dim lHDC As Long
Dim lIndex As Long
Dim lCOlour As Long
Dim lDIff As Long
Dim lFilLColour As Long
Dim bDisabled As Boolean
Dim bSelected As Boolean
Dim bChecked As Boolean
Dim bIsTOpLevel As Boolean
Dim hBrush As Long
Dim tP As POINTAPI
Dim tB As RECT
Dim tC As RECT
Dim tOB As RECT
Dim tS As RECT
Dim tFR As RECT
Dim tIR As RECT
Dim tFC As RECT
Dim sText As String
Dim x As Long

    CopyMemory tDI, ByVal lParam, Len(tDI)
    lIndex = (plGetIndexForId(tDI.itemID))
    If (lIndex <> 0) Then
        ' Debug.Print "Found item to draw."
        lHDC = tDI.hDC
        SetBkMode lHDC, TRANSPARENT
        
        bDisabled = ((tDI.itemState And ODS_DISABLED) = ODS_DISABLED) Or ((tDI.itemState And ODS_GRAYED) = ODS_GRAYED)
        bSelected = ((tDI.itemState And ODS_SELECTED) = ODS_SELECTED)
        bChecked = ((tDI.itemState And ODS_CHECKED) = ODS_CHECKED)
        bIsTOpLevel = pbIsTopLevelmenu(lIndex)
        
        tP.x = tDI.rcItem.left
        tP.y = tDI.rcItem.tOp + 1
        
        ' Store the rectangle into tOB so we can draw an
        ' outer border if this is a top level item:
        CopyMemory tOB, tDI.rcItem, LenB(tB)
                
        If (bDisabled) Then
            If (Trim$(m_tMI(lIndex).sCaption = "-")) Then
                ' If this is a separator, then draw the separator line:
                tS.left = tP.x
                tS.Right = tDI.rcItem.Right
                tS.tOp = tP.y + 1
                tS.Bottom = tS.tOp + 2
                DrawEdge lHDC, tS, EDGE_ETCHED, BF_TOP
            Else
                ' Get the position to output the text:
                pGetTextPosition lHDC, lIndex, bIsTOpLevel, tDI.rcItem
                
                ' Draw the text grayed:
                With tDI.rcItem
                    .left = .left + 1
                    .tOp = .tOp + 1
                End With
                SetTextColor lHDC, GetSysColor(COLOR_BTNHIGHLIGHT)
                pDrawMenuCaption lHDC, lIndex, tDI.rcItem
                With tDI.rcItem
                    .left = .left - 1
                    .tOp = .tOp - 1
                End With
                SetTextColor lHDC, GetSysColor(COLOR_BTNSHADOW)
                pDrawMenuCaption lHDC, lIndex, tDI.rcItem
                
                SetTextColor lHDC, GetSysColor(COLOR_MENUTEXT)
            End If
        Else
            If (Trim$(m_tMI(lIndex).sCaption = "-")) Then
                ' Draw a separator line:
                If (bIsTOpLevel) Then
                    ' We draw nothing - a separator at
                    ' the top level just leaves a space.
                    Debug.Print "Separator at top level"
                Else
                    ' We draw the separator line:
                    tS.left = tP.x
                    tS.tOp = tP.y + 1
                    tS.Bottom = tS.tOp + 2
                    tS.Right = tDI.rcItem.Right
                    DrawEdge lHDC, tS, EDGE_ETCHED, BF_TOP
                End If
            Else
            
                ' Set the back colour and text colour for the menu item:
                If bSelected And Not (bIsTOpLevel) Then
                    lFilLColour = GetSysColor(COLOR_HIGHLIGHT)
                    lCOlour = GetSysColor(COLOR_HIGHLIGHTTEXT)
                Else
                    lFilLColour = GetSysColor(COLOR_MENU)
                    lCOlour = GetSysColor(COLOR_MENUTEXT)
                End If
                CopyMemory tFR, tDI.rcItem, LenB(tFR)
                If (m_tMI(lIndex).lIconIndex > -1) Or (bChecked) Then
                    ' Erase the icon background:
                    CopyMemory tIR, tDI.rcItem, LenB(tIR)
                    tIR.Right = tIR.left + m_lMenuItemHeight - 1
                    hBrush = CreateSolidBrush(GetSysColor(COLOR_MENU))
                    FillRect lHDC, tIR, hBrush
                    DeleteObject hBrush
                    tFR.left = tFR.left + m_lMenuItemHeight
                End If
                hBrush = CreateSolidBrush(lFilLColour)
                FillRect lHDC, tFR, hBrush
                DeleteObject hBrush
                SetTextColor lHDC, lCOlour
                
                ' Get the position to output the text:
                pGetTextPosition lHDC, lIndex, bIsTOpLevel, tDI.rcItem
                pDrawMenuCaption lHDC, lIndex, tDI.rcItem
                                
                If (bIsTOpLevel) And (bSelected) Then
                    ' We draw a sunken box around selected
                    ' top level menu items:
                    tOB.Right = tOB.Right - 1
                    tOB.Bottom = tOB.Bottom
                    DrawEdge lHDC, tOB, BDR_SUNKENOUTER, BF_RECT
                End If
            End If
        End If
        
        If (bChecked) Then
            ' We draw a sunken box around the checked item:
            tB.left = tP.x
            tB.tOp = tP.y + 1
            tB.Right = tB.left + m_lMenuItemHeight - 1
            tB.Bottom = tB.tOp + m_lMenuItemHeight - 2
            DrawEdge lHDC, tB, BDR_SUNKENOUTER, BF_RECT
            ' If we're not disabled, we should fill the background.
            ' If we're selected, we use BTNFACE, otherwise we use
            ' BTNHIGHLIGHT (although really we should be dithering
            ' it... but its too hard to do right now!)>
            If Not (bDisabled) Then
                If (bSelected) Then
                    hBrush = CreateSolidBrush(GetSysColor(COLOR_BTNFACE))
                Else
                    hBrush = CreateSolidBrush(GetSysColor(COLOR_BTNHIGHLIGHT))
                End If
                CopyMemory tFC, tB, LenB(tFC)
                With tFC
                    .left = .left + 1
                    .tOp = .tOp + 1
                    .Right = .Right - 2
                    .Bottom = .Bottom - 2
                End With
                FillRect lHDC, tFC, hBrush
                DeleteObject hBrush
            End If
            
            ' If we don't have an icon, we ought to draw the tick icon
            ' inside it:
            If (m_tMI(lIndex).lIconIndex = -1) Then
                If (bIsTOpLevel) Then
                    x = tP.x
                Else
                    x = tP.x + (m_lMenuItemHeight - 18) \ 2
                End If
                If (bDisabled) Then
                    ImageListDrawIconDisabled lHDC, m_hIml, m_lTickIconIndex, x + 1, tP.y + 3, 16
                Else
                    ImageListDrawIcon lHDC, m_hIml, m_lTickIconIndex, x + 1, tP.y + 3, , (bDisabled)
                End If
            End If
        ElseIf (bSelected) And Not (bIsTOpLevel) And Not (bDisabled) Then
            ' We should draw a raised box around the icon:
            If (m_tMI(lIndex).lIconIndex > -1) Then
                tB.left = tP.x
                tB.tOp = tP.y - 1
                tB.Right = tB.left + m_lMenuItemHeight - 1
                tB.Bottom = tB.tOp + m_lMenuItemHeight
                DrawEdge lHDC, tB, BDR_RAISEDINNER, BF_RECT
            End If
        End If

        If (m_tMI(lIndex).lIconIndex > -1) Then
            If (bIsTOpLevel) Then
                x = tP.x
            Else
                x = tP.x + (m_lMenuItemHeight - 18) \ 2
            End If
            If (bDisabled) Then
                ImageListDrawIconDisabled lHDC, m_hIml, m_tMI(lIndex).lIconIndex, x + 1, tP.y + 3, 16
            Else
                ImageListDrawIcon lHDC, m_hIml, m_tMI(lIndex).lIconIndex, x + 1, tP.y + 3, , (bDisabled)
            End If
        End If
        
    Else
        Debug.Print "Failed to find item to draw.", tDI.itemID, lIndex
    End If
End Sub
Private Sub pDrawMenuCaption( _
        ByVal lHDC As Long, _
        ByVal lIndex As Long, _
        ByRef tR As RECT _
    )
Dim sText As String
Dim tSR As RECT

    sText = Trim$(m_tMI(lIndex).sCaption)
    DrawText lHDC, sText, Len(sText), tR, DT_LEFT
    
    sText = Trim$(m_tMI(lIndex).sShortCutDisplay)
    If (sText <> "") Then
        CopyMemory tSR, tR, LenB(tR)
        tSR.left = m_tMI(lIndex).lShortCutStartPos
        DrawText lHDC, sText, Len(sText), tSR, DT_LEFT
    End If
End Sub
Private Function pbIsTopLevelmenu( _
        ByVal lIndex As Long _
    ) As Boolean
    pbIsTopLevelmenu = (m_tMI(lIndex).hMenu = GetMenu(m_hWndParent))
End Function
Private Function pGetTextPosition( _
        ByVal lHDC As Long, _
        ByVal lIndex As Long, _
        ByVal bIsTOpLevel As Boolean, _
        ByRef rcItem As RECT _
    )
Dim tC As RECT
Dim lDIff As Long
Dim lMenuHeight As Long

    If (bIsTOpLevel) Then
        lMenuHeight = GetSystemMetrics(SM_CYMENU) - 2 ' Allow for border
    Else
        lMenuHeight = m_lMenuItemHeight
    End If
    
    ' Determine the size of the text to draw:
    DrawText lHDC, m_tMI(lIndex).sCaption, Len(m_tMI(lIndex).sCaption), tC, DT_CALCRECT
    
    ' We want to centre the text vertically:
    lDIff = lMenuHeight - (tC.Bottom - tC.tOp)
    If (lDIff > 0) Then
        rcItem.tOp = rcItem.tOp + lDIff \ 2
    End If
    
    ' Now move the left position of the text
    ' across to accommodate icon/selection rectangle:
    If (bIsTOpLevel) Then
        ' If its a top level item, then move across
        ' to accommodate the border.  Additionally, if
        ' there is an icon, move across to accomodate
        ' the icon:
        If (m_tMI(lIndex).lIconIndex > -1) Then
            rcItem.left = rcItem.left + 18
        Else
            rcItem.left = rcItem.left + 4
        End If
    Else
        ' All normal menu items are indented by 26 to
        ' accomodate icon & checked surround for icon:
        rcItem.left = rcItem.left + 26
    End If
    
End Function
Private Function plGetIndexForId( _
        ByVal lItemId As Long _
    ) As Long
Dim l As Long
Dim lIndex As Long
    'Debug.Print "Finding Index:"
    'Debug.Print lItemId
    lIndex = 0
    For l = 1 To m_iMenuCount
        'Debug.Print "    Index at l = " & m_tMI(l).lId
        If (m_tMI(l).lActualID = lItemId) Then
            lIndex = l
            Exit For
        End If
    Next l
    plGetIndexForId = lIndex
End Function
Private Sub pGetTheMenuFont()

End Sub
Private Sub MeasureItem( _
        ByVal lItemId As Long, _
        ByRef lWidth As Long, _
        ByRef lHeight As Long _
    )
Dim lIndex As Long
Dim tR As RECT
Dim bDontEvalWidth As Long
Dim bIsTOpLevel As Long
Dim sLongestCaption As String
Dim sLongestShortCut As String
Dim l As Long
Dim lItemsOnMenu() As Long
Dim lItemCount As Long
Dim hMenu As Long

    pGetTheMenuFont

    lIndex = plGetIndexForId(lItemId)
    If (lIndex <> 0) Then
        bIsTOpLevel = pbIsTopLevelmenu(lIndex)
        If (bIsTOpLevel) Then
            lHeight = GetSystemMetrics(SM_CYMENU)
        Else
            lHeight = m_lMenuItemHeight
        End If
        ' Determine the width of the item:
        If bIsTOpLevel Then
            'Debug.Print "Top Level"
            If Trim$(m_tMI(lIndex).sCaption) = "-" Or Trim$(m_tMI(lIndex).sCaption) = "" Then
                lWidth = 4
                bDontEvalWidth = True
            Else
                If (m_tMI(lIndex).lIconIndex > -1) Then
                    lWidth = 18
                Else
                    lWidth = 0
                End If
            End If
            'Debug.Print lWidth
        Else
            If Trim$(m_tMI(lIndex).sCaption = "-") Then
                lHeight = 6
                lWidth = 32
                bDontEvalWidth = True
            Else
                lWidth = 32
            End If
        End If
        
        If Not (bDontEvalWidth) Then
            If bIsTOpLevel Then
                ' For top level items we evaluate the width of
                ' the actual text item only:
                DrawText m_HDC, m_tMI(lIndex).sCaption, Len(m_tMI(lIndex).sCaption), tR, DT_CALCRECT
                lWidth = lWidth + tR.Right
            Else
                ' If we have shortcuts at this level (CTRL-C etc), we need
                ' to evaluate the widths of all text items in this menu,
                ' then set the width to be the width of the longest text +
                ' the longest shortcut item:
                hMenu = m_tMI(lIndex).hMenu
                For l = 1 To m_iMenuCount
                    If (m_tMI(l).hMenu = hMenu) Then
                        If Len(m_tMI(l).sCaption) > Len(sLongestCaption) Then
                            sLongestCaption = m_tMI(l).sCaption
                        End If
                        If (Len(m_tMI(l).sShortCutDisplay) > Len(sLongestShortCut)) Then
                            sLongestShortCut = m_tMI(l).sShortCutDisplay
                        End If
                        lItemCount = lItemCount + 1
                        ReDim Preserve lItemsOnMenu(1 To lItemCount) As Long
                        lItemsOnMenu(lItemCount) = l
                    End If
                Next l
                DrawText m_HDC, sLongestCaption, Len(sLongestCaption), tR, DT_CALCRECT
                lWidth = lWidth + tR.Right
                If (sLongestShortCut <> "") Then
                    DrawText m_HDC, sLongestShortCut, Len(sLongestShortCut), tR, DT_CALCRECT
                    lWidth = lWidth + 8
                    For l = 1 To lItemCount
                        m_tMI(lItemsOnMenu(l)).lShortCutStartPos = lWidth
                    Next l
                    lWidth = lWidth + tR.Right
                End If
            End If
        End If
        'Debug.Print "Width " & lWidth
    
    End If
    
End Sub
Property Get IDForIndex( _
        ByVal vKey As Variant _
    ) As Long
Attribute IDForIndex.VB_Description = "Returns the ID used to identify a menu item."
Dim lIndex As Long
    lIndex = MenuIndex(vKey)
    If (lIndex > 0) Then
        IDForIndex = m_tMI(lIndex).lActualID
    End If
End Property
Public Function AddItem( _
        ByVal sCaption As String, _
        Optional ByVal sKey As String = "", _
        Optional ByVal sHelptext As String = "", _
        Optional ByVal lItemData As Long = 0, _
        Optional ByVal lParentIndex As Long = 0, _
        Optional ByVal lIconIndex As Long = -1, _
        Optional ByVal bChecked As Boolean = False, _
        Optional ByVal bEnabled As Boolean = True _
    ) As Long
Attribute AddItem.VB_Description = "Adds a new menu to VBs menu list."
Dim lId As Long
    If (pbIsValidKey(sKey)) Then
        m_iMenuCount = m_iMenuCount + 1
        ReDim Preserve m_tMI(1 To m_iMenuCount) As tMenuItem
        lId = plGetNewID()
        With m_tMI(m_iMenuCount)
            .lId = lId
            .lActualID = lId
            .sCaption = sCaption
            .sAccelerator = psExtractAccelerator(sCaption)
            .sHelptext = sHelptext
            .lIconIndex = lIconIndex
            .lParentId = m_tMI(lParentIndex).lActualID
            .lItemData = lItemData
            .bChecked = bChecked
            .bEnabled = bEnabled
            .bCreated = True
        End With
        pAddNewMenuItem m_tMI(m_iMenuCount)
        AddItem = m_iMenuCount
    End If
End Function
Private Function plGetNewID() As Long
Dim lId As Long

    If (m_lLastMaxId < m_iMenuCount) Then
        m_lLastMaxId = m_iMenuCount
    Else
        m_lLastMaxId = m_lLastMaxId + 1
    End If
    lId = m_lLastMaxId
    Do Until (pbIDIsUnique(lId))
        lId = lId + 1
        m_lLastMaxId = lId
    Loop
    plGetNewID = lId
End Function
Private Function pbIDIsUnique( _
        ByVal lId As Long _
    ) As Boolean
Dim bFound As Boolean
Dim lMenu As Long
    For lMenu = 1 To m_iMenuCount
        If (m_tMI(lMenu).lActualID = lId) Then
            bFound = True
            Exit For
        End If
    Next lMenu
    pbIDIsUnique = Not (bFound)
End Function
Private Function psExtractAccelerator( _
        ByVal sCaption As String _
    )
Dim i As Long

    For i = 1 To Len(sCaption)
        If (Mid$(sCaption, i, 1) = "&") Then
            If (i < Len(sCaption)) Then
                psExtractAccelerator = UCase$(Mid$(sCaption, (i + 1), 1))
            End If
            Exit For
        End If
    Next i
End Function
Private Sub pAddNewMenuItem( _
        ByRef tMI As tMenuItem _
    )
Dim tMII As MENUITEMINFO
Dim hMenu As Long
Dim lPIndex As Long
Dim lFlags As Long
Dim lR As Long

    ' Find out where we're adding this item:
    With tMI
        If (.lParentId = 0) Then
            ' This is a new top level menu item:
            hMenu = GetMenu(m_hWndParent)
        Else
            ' We are adding to an existing menu:
            ' First we need to determine if there is already a sub menu for the parent item:
            lPIndex = plGetIndexForId(tMI.lParentId)
            If (lPIndex = 0) Then
                Debug.Print " *** Couldn't find parent... *** "
            Else
                ' Determine if the parent menu has a sub-menu:
                tMII.fMask = MIIM_SUBMENU
                tMII.cbSize = LenB(tMII)
                GetMenuItemInfo m_tMI(lPIndex).hMenu, m_tMI(lPIndex).lActualID, False, tMII
                hMenu = tMII.hSubMenu
                If (hMenu = 0) Then
                    ' We don't have a sub menu for this item so we're
                    ' going to have to add one:
                    ' Debug.Print "Adding new sub-menu:"
                    
                    ' Create the new menu item and store it's handle so we can clear up
                    ' again later:
                    hMenu = CreatePopupMenu()
                    If (hMenu = 0) Then
                        Debug.Print " *** Failed to create sub menu *** "
                    Else
                    m_lSubMenuCount = m_lSubMenuCount + 1
                    ReDim Preserve m_hSubMenus(1 To m_lSubMenuCount) As Long
                    m_hSubMenus(m_lSubMenuCount) = hMenu
                    
                    ' Now set the parent item so it has a popup menu:
                    lFlags = plMenuFlags(lPIndex)
                    lFlags = (lFlags Or MF_OWNERDRAW) And Not MF_STRING
                    lFlags = lFlags Or MF_POPUP
                    lFlags = lFlags And Not MF_BYPOSITION Or MF_BYCOMMAND
                    
                    lR = ModifyMenuByLong(m_tMI(lPIndex).hMenu, m_tMI(lPIndex).lActualID, lFlags, hMenu, m_tMI(lPIndex).lActualID)
                    If (lR = 0) Then
                        Debug.Print "Failed to modify menu to add the sub menu " & GetLastError()
                    End If
                    
                    ' WHen you add a sub menu to an item, its id becomes the sub menu handle:
                    m_tMI(lPIndex).lActualID = hMenu
                    tMI.lParentId = hMenu
                    End If
                End If
            End If
        End If
        
        If (hMenu <> 0) Then
            lFlags = plMenuFlags(m_iMenuCount)
            lFlags = (lFlags Or MF_OWNERDRAW) And Not MF_STRING Or MF_BYPOSITION
            lR = AppendMenuBylong(hMenu, lFlags, tMI.lId, tMI.lId)
            If (lR = 0) Then
                Debug.Print "Failed to add new Menu item"
            End If
        End If
        
        ' Store the hMenu for this item:
        .hMenu = hMenu
        
    End With
End Sub
Public Sub Clear()
Attribute Clear.VB_Description = "Clears all the menus in the item."
Attribute Clear.VB_MemberFlags = "40"
    m_iMenuCount = 0
    Erase m_tMI
End Sub
Property Get Count() As Integer
Attribute Count.VB_Description = "Returns the total number of items in the menu."
    Count = m_iMenuCount
End Property
Private Sub pRemoveMenuItems( _
        ByVal hMenu As Long _
    )
Dim lMenu As Long
    For lMenu = 1 To m_lSubMenuCount
        DestroyMenu m_hSubMenus(lMenu)
    Next lMenu
    
End Sub
Private Function plMenuFlags( _
        ByVal lIndex As Long _
    )
Dim lFlags As Long
    With m_tMI(lIndex)
        If (.bChecked) Then
            lFlags = lFlags Or MF_CHECKED
        Else
            lFlags = lFlags Or MF_UNCHECKED
        End If
        If (.bEnabled) Then
            lFlags = lFlags Or MF_ENABLED
        Else
            lFlags = lFlags Or MF_GRAYED
        End If
        If (Trim$(.sCaption) = "-") Then
            lFlags = lFlags Or MF_SEPARATOR
        End If
    End With
    plMenuFlags = lFlags
End Function

Public Sub SubClassMenu( _
        Optional ByVal oForm As Object = Nothing _
    )
Attribute SubClassMenu.VB_Description = "Initialises the popup menu control against the parent form."
Dim hMenu As Long
Dim tVBInfo() As tVBMenuInfo
Dim iVBMenuCount As Long
Dim i As Long
Dim lIndex As Long

    Clear
    
    If (m_hWndParent <> 0) Then
        
        If Not (oForm Is Nothing) Then
            ' Loop through the form object to find the menus.
            ' Store their caption and name.  We use this to
            ' set the key in the internal array based on their name:
            For i = 0 To oForm.Controls.Count - 1
                If (TypeOf oForm.Controls(i) Is Menu) Then
                    iVBMenuCount = iVBMenuCount + 1
                    ReDim Preserve tVBInfo(1 To iVBMenuCount) As tVBMenuInfo
                    tVBInfo(iVBMenuCount).sName = oForm.Controls(i).Name
                    tVBInfo(iVBMenuCount).sCaption = oForm.Controls(i).Caption
                    On Error Resume Next
                    lIndex = oForm.Controls(i).Index
                    If (Err.Number = 0) Then
                        tVBInfo(iVBMenuCount).bHasIndex = True
                        tVBInfo(iVBMenuCount).iIndex = lIndex
                    End If
                    Err.Clear
                    tVBInfo(iVBMenuCount).bUsed = oForm.Controls(i).Visible
                End If
            Next i
        End If
        
        hMenu = GetMenu(m_hWndParent)
        pUpdateMenuItems hMenu, 0, False
        
        ' Now try to associate VB menus with the ones we've just updated:
        If (iVBMenuCount > 0) Then
            i = 0
            For lIndex = 1 To m_iMenuCount
                i = i + 1
                Do While Not (tVBInfo(i).bUsed)
                    i = i + 1
                    If (i > iVBMenuCount) Then
                        Exit Do
                    End If
                Loop
                If (i > iVBMenuCount) Then
                    Exit For
                End If
                ' These should match!
                ' Debug.Print tVBInfo(i).sCaption, m_tMI(lIndex).sCaption
                m_tMI(lIndex).sKey = tVBInfo(i).sName
                If (tVBInfo(i).bHasIndex) Then
                    m_tMI(lIndex).sKey = m_tMI(lIndex).sKey & "(" & tVBInfo(i).iIndex & ")"
                End If
            Next lIndex
        End If
        
        ' Draw the menu:
        DrawMenuBar m_hWndParent
    End If
    
End Sub

Private Sub pUpdateMenuItems( _
        ByVal hMenu As Long, _
        ByVal lParentId As Long, _
        ByVal bUpdate As Boolean _
    )
Dim lCount As Long
Dim lMenu As Long
Dim hSubMenu As Long
        
    lCount = GetMenuItemCount(hMenu)
    For lMenu = 1 To lCount
        pAddMenuItem hMenu, lMenu, lParentId, bUpdate
        hSubMenu = GetSubMenu(hMenu, (lMenu - 1))
            'Debug.Print hSubMenu
        If (hSubMenu <> 0) Then
            ' Recurse for the sub menus:
            pUpdateMenuItems hSubMenu, hSubMenu, bUpdate
        End If
    Next lMenu

End Sub
Private Sub pAddMenuItem( _
        ByVal hMenu As Long, _
        ByVal lPosition As Long, _
        ByVal lParentId As Long, _
        ByVal bUpdate As Boolean _
    )
Dim tMI As MENUITEMINFO
Dim lFlags As Long
Dim lR As Long
Dim bTopMenu As Boolean
Dim lIndex As Long
Dim lId As Long
Dim bAlreadyHave As Boolean
    
        'Debug.Print "Adding"
    
    ' Get information about the current menu item:
    ' Do we already have this menu?
    If (bUpdate) Then
        lId = GetMenuItemID(hMenu, (lPosition - 1))
        lIndex = IndexForId(lId)
        If (lIndex > 0) Then
            bAlreadyHave = True
        End If
    End If
    
    If Not (bAlreadyHave) Then
        tMI.fMask = MIIM_DATA Or MIIM_ID Or MIIM_STATE Or MIIM_SUBMENU Or MIIM_TYPE
        tMI.cch = 127
        tMI.dwTypeData = String$(128, 0)
        tMI.cbSize = LenB(tMI)
        GetMenuItemInfo hMenu, (lPosition - 1), 1, tMI
            
        ' Add a this item to the internal menu item array:
        m_iMenuCount = m_iMenuCount + 1
        ReDim Preserve m_tMI(1 To m_iMenuCount) As tMenuItem
        lIndex = m_iMenuCount
    End If
    
    With m_tMI(lIndex)
        .bIsAVBMenu = True
        .lIconIndex = -1     ' Start off with no icon
        .bChecked = ((tMI.fState And MFS_CHECKED) = MFS_CHECKED)
        .bEnabled = Not ((tMI.fState And MFS_DISABLED) = MFS_DISABLED)
        .hMenu = hMenu
        .lId = tMI.wID
        .lActualID = tMI.wID
        .lParentId = lParentId
    End With
    pSetMenuCaption lIndex, left$(tMI.dwTypeData, tMI.cch), ((tMI.fType And MF_SEPARATOR) = MF_SEPARATOR)
        
    ' Get the flag equivalent for this menu item:
    lFlags = plMenuFlags(lIndex)
    ' Set it to owner draw:
    lFlags = lFlags Or MF_OWNERDRAW
    ' Ensure the string flag is removed:
    lFlags = lFlags And Not MF_STRING
    ' If there is a popup menu, make sure we keep it:
    If (tMI.hSubMenu <> 0) Then
        lFlags = lFlags Or MF_POPUP
    End If
    ' Modifying by position:
    lFlags = lFlags Or MF_BYPOSITION
    
    ' Now set the menu to the owner draw version:
    lR = ModifyMenuByLong(hMenu, (lPosition - 1), lFlags, m_tMI(lIndex).lActualID, m_tMI(lIndex).lActualID)
        ' This really shouldn't happen:
        If (lR = 0) Then
            Debug.Print "ModifyMenu failed:" & GetLastError()
        End If
End Sub
Private Sub pSetMenuCaption( _
        ByVal iItem As Long, _
        ByVal sCaption As String, _
        ByVal bSeparator As Boolean _
    )
Dim sCap As String
Dim sShortCut As String
Dim iPos As Integer

    If (bSeparator) Then
        m_tMI(iItem).sCaption = "-"
    Else
        ' Check if we have a shortcut to the menu item:
        iPos = InStr(sCaption, vbTab)
        If (iPos <> 0) Then
            sCap = left$(sCaption, (iPos - 1))
            ' Extract the ctrl key item:
            sShortCut = Mid$(sCaption, (iPos + 1))
            pParseMenuShortcut iItem, sShortCut
        Else
            sCap = sCaption
        End If
        m_tMI(iItem).sAccelerator = psExtractAccelerator(sCap)
        m_tMI(iItem).sCaption = sCap
    End If

End Sub
    
Private Sub pParseMenuShortcut( _
        ByVal iItem As Long, _
        ByVal sShortCut As String _
    )
Dim bNotFKey As Boolean
Dim iPos As Integer
Dim iLen As Integer
Dim sKey As String
Dim SkeyNum As String

    m_tMI(iItem).iShortCutShiftMask = 0
    m_tMI(iItem).iShortCutShiftKey = 0
    m_tMI(iItem).sShortCutDisplay = sShortCut
    
    If (sShortCut <> "") Then
        If (InStr(sShortCut, "Ctrl")) Then
            m_tMI(iItem).iShortCutShiftMask = vbCtrlMask
            bNotFKey = True
        End If
        If (InStr(sShortCut, "Shift")) Then
            m_tMI(iItem).iShortCutShiftMask = m_tMI(iItem).iShortCutShiftMask Or vbShiftMask
            bNotFKey = True
        End If
        
        If (bNotFKey) Then
            ' Find the last + and get the key:
            iLen = Len(sShortCut)
            iPos = iLen
            Do While Mid$(sShortCut, iPos, 1) <> "+" And iPos > 1
                iPos = iPos - 1
            Loop
            sKey = Mid$(sShortCut, iPos)
            If (Len(sKey) = 1) Then
                m_tMI(iItem).iShortCutShiftKey = Asc(sKey)
            Else
                ' Check for F key, Space, Backspace, Del
                
            End If
        Else
            ' Parse the Fkey:
            iPos = InStr(sShortCut, "F")
            If (iPos <> 0) Then
                SkeyNum = Mid$(sShortCut, (iPos + 1))
                m_tMI(iItem).iShortCutShiftKey = vbKeyF1 + Val(SkeyNum) - 1
            End If
        End If
    End If
    
End Sub
    
Private Property Let ISubclass_MsgResponse(ByVal RHS As SSubTimer.EMsgResponse)
    m_emr = RHS
End Property

Private Property Get ISubclass_MsgResponse() As SSubTimer.EMsgResponse
    ISubclass_MsgResponse = m_emr
End Property

Private Function ISubclass_WindowProc(ByVal hwnd As Long, ByVal iMsg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long
    Dim lMenuId As Long, hMenu As Long, lItem As Long
Dim lMenuCount As Long
Dim lHiWord As Long
Dim bEnabled As Boolean, bSeparator As Boolean
Dim bFound As Boolean
Dim bNoDefault As Boolean
Dim iChar As Integer
Dim lFlag As Long

    Select Case iMsg
    
    ' Handle Menu Select events:
    Case WM_MENUSELECT
        ' Extract the menu id and flags for the selected
        ' menu item:
        lHiWord = wParam \ &H10000
        lMenuId = wParam And &HFFFF&
        
        ' MenuId 0 corresponds to a separator on the system
        ' menu:
        If (lMenuId <> 0) Then
            
            ' Extract separator & enabled/disabled from the flags
            ' stored in the High Word of wParam:
            bSeparator = ((lHiWord And MF_SEPARATOR) = MF_SEPARATOR)
            bEnabled = ((lHiWord And MF_DISABLED) = MF_DISABLED) Or ((lHiWord And MF_GRAYED) = MF_GRAYED)
            
            ' Menu handle is passed in as lParam:
            hMenu = lParam
            
            ' Now check if the message is a menu item higlight,
            ' or whether it is indicating exit from the menu:
            lMenuCount = GetMenuItemCount(hMenu)
            For lItem = 0 To lMenuCount - 1
                If (lMenuId = GetMenuItemID(hMenu, lItem)) Then
                    bFound = True
                    Exit For
                End If
            Next lItem
            
            ' Raise a highlight or menu exit as required:
            If (bFound) Then
                RaiseHighlightEvent lMenuId
            Else
                RaiseMenuExitEvent
            End If
            
        End If
        
        ' Let the MENU_SELECT event filter through wherever
        ' else it is going:
        m_emr = emrPostProcess
        
        
    ' Handle menu click events:
    Case WM_COMMAND
        Debug.Print "Got a WM_COMMAND"
        
        ' Commands from menus are identified by an lParam of 0
        ' (otherwise it is set the hWnd of the control):
        If (lParam = 0) Then
            ' Low order word of the wParam item is the menu item id:
            lMenuId = (wParam And &HFFFF&)
            
            Debug.Print "ID: " & lMenuId
            If (RaiseClickEvent(lMenuId)) Then
                ' Don't send on the WM_COMMAND if the item
                ' wasn't a VB menu, it might interfere
                ' with some other control items!
                m_emr = emrConsume
            Else
                ' Otherwise allow the message to parse through
                ' to the click event on the VB menu so your old
                ' code continues to work:
                m_emr = emrPostProcess
            End If
        Else
            m_emr = emrPostProcess
        End If
        
    
    ' Handle system menu click events:
    Case WM_SYSCOMMAND
        'Debug.Print "Got a SYSCOMMAND item"
        
        ' Check if the item is a system menu command:
        If (pbIdIsSysMenuId(wParam)) Then
            ' If it is, send the event:
            RaiseEvent SystemMenuClick(wParam)
        End If
        
        ' Always let the message do its normal work:
        m_emr = emrPostProcess
        
    
    ' Draw Menu items:
    Case WM_DRAWITEM
        'Debug.Print "Got a draw item"
        DrawItem lParam, wParam
        
    
    ' Measure Menu items prior to drawing them:
    Case WM_MEASUREITEM
        ' Debug.Print "Measure item"
        Dim tMis As MEASUREITEMSTRUCT
        CopyMemory tMis, ByVal lParam, Len(tMis)
        ' Get the required width & height:
        MeasureItem tMis.itemID, tMis.itemWidth, tMis.itemHeight
        ' Put the new items back into the structure:
        CopyMemory ByVal lParam, tMis, Len(tMis)
        ISubclass_WindowProc = 1
        m_emr = emrConsume
        
    
    ' Handle accelerator (&key) messages in the menu:
    Case WM_MENUCHAR
        ' Check that this is my menu:
        lFlag = wParam \ &H10000
        If ((lFlag And MF_SYSMENU) <> MF_SYSMENU) Then
            hMenu = lParam
            iChar = (wParam And &HFFFF&)
            ' Debug.Print hMenu, Chr$(iChar)
            ' See if this corresponds to an accelerator on the menu:
            ISubclass_WindowProc = plParseMenuChar(hMenu, iChar)
            m_emr = emrConsume
        End If
        
        
    Case WM_INITMENUPOPUP
        ' Check the sys menu flag:
        If (lParam \ &H10000) > 0 Then
            ' System menu.
        Else
            hMenu = wParam
            ' Find the item which is the parent
            ' of this popup menu:
            RaiseInitMenuEvent hMenu
        End If
        m_emr = emrPostProcess
        
    
    Case WM_WININICHANGE
        ' First ensure we have the correct font:
        pSelectMenuFont
        ' Now allow the event to be responded
        ' to in the form
        RaiseEvent WinIniChange
        ' Make sure we pass the message on for
        ' default processing!
        m_emr = emrPostProcess
    
    End Select

End Function
Private Sub RaiseInitMenuEvent( _
        ByVal hMenu As Long _
    )
Dim lIndex As Long
Dim lParentId As Long
Dim bFound As Boolean

    ' Firstly, we need to find the index of an item
    ' in hMenu:
    For lIndex = m_iMenuCount To 1 Step -1
        If (m_tMI(lIndex).hMenu = hMenu) Then
            lParentId = m_tMI(lIndex).lParentId
            bFound = True
        End If
        If (bFound) Then
            If (m_tMI(lIndex).lActualID = lParentId) Then
                RaiseEvent InitPopupMenu(lIndex)
                Exit For
            End If
        End If
    Next lIndex
                
End Sub
Private Sub pCreateSubClass(hWndA As Long)
    AttachMessage Me, hWndA, WM_MENUSELECT
    AttachMessage Me, hWndA, WM_MEASUREITEM
    AttachMessage Me, hWndA, WM_DRAWITEM
    AttachMessage Me, hWndA, WM_COMMAND
    AttachMessage Me, hWndA, WM_MENUCHAR
    AttachMessage Me, hWndA, WM_SYSCOMMAND
    AttachMessage Me, hWndA, WM_INITMENUPOPUP
    AttachMessage Me, hWndA, WM_WININICHANGE
End Sub

Private Sub pDestroySubClass()
    If (m_hWndParent <> 0) Then
        DetachMessage Me, m_hWndParent, WM_MENUSELECT
        DetachMessage Me, m_hWndParent, WM_MEASUREITEM
        DetachMessage Me, m_hWndParent, WM_DRAWITEM
        DetachMessage Me, m_hWndParent, WM_COMMAND
        DetachMessage Me, m_hWndParent, WM_MENUCHAR
        DetachMessage Me, m_hWndParent, WM_SYSCOMMAND
        DetachMessage Me, m_hWndParent, WM_INITMENUPOPUP
        DetachMessage Me, m_hWndParent, WM_WININICHANGE
    End If
    m_hWndParent = 0
End Sub
Private Sub UserControl_Initialize()
    Debug.Print "Initialise"
    m_lNextSysMenuID = WM_MENUBASE
    m_lMenuItemHeight = 22
    Set m_cNCM = New cNCMetrics
End Sub

Private Sub UserControl_Paint()
Dim lHDC As Long
Dim tR As RECT
    tR.Right = 40
    tR.Bottom = 40
    lHDC = UserControl.hDC
    DrawEdge lHDC, tR, EDGE_RAISED, BF_RECT
    BitBlt lHDC, 4, 4, 32, 32, m_HDC, 0, 0, SRCCOPY
End Sub

Private Sub UserControl_ReadProperties(PropBag As PropertyBag)

    Debug.Print "ReadProperties"
    
    ' At the ReadProperties we are now sited
    ' and have a fully usable UserControl
    ' object.
    
    If (UserControl.Ambient.UserMode) Then
        ' Only do the subclassing stuff whilst
        ' we are in run mode.  Makes it easier
        ' to debug, if nothing else...
        m_hWndParent = UserControl.Parent.hwnd
        ' Make a HDC to allow us to evaluate the
        ' size of menu items.
        m_HDC = CreateCompatibleDC(UserControl.hDC)
        ' Select the menu font into it:
        pSelectMenuFont
        ' Create work dc's so we can create disabled icons
        ' from normal icons in the image list:
        ImageListMakeWorkDCS m_HDC, 16
        ' Start subclassing:
        Debug.Print "Start subclassing"
        pCreateSubClass m_hWndParent
    Else
        ' We don't draw when we're in run mode so
        ' only do it when not in run mode:
        pMakeDisplay
    End If
End Sub
Private Sub pSelectMenuFont()
Dim tM As RECT
    ' If we have already selected the font,
    ' then remove it from the DC:
    If (m_hFntOld <> 0) Then
        SelectObject m_HDC, m_hFntOld
    End If
    ' Get the metrics.  This will delete
    ' the hFont for menu:
    m_cNCM.GetMetrics
    ' Select the latest version of the menu font
    ' into the DC, storing what was there before:
    m_hFntOld = SelectObject(m_HDC, m_cNCM.FontHandle(MenuFOnt))
    
    ' Determine what height to make the menu items:
    DrawText m_HDC, "yY", 2, tM, DT_CALCRECT
    If (tM.Bottom - tM.tOp) > 18 Then
        m_lMenuItemHeight = tM.Bottom - tM.tOp + 6
    Else
        m_lMenuItemHeight = 22
    End If
    DrawMenuBar m_hWndParent
    
End Sub
Private Sub pMakeDisplay()
Dim hInst As Long
    m_HDC = CreateCompatibleDC(UserControl.hDC)
    If (m_HDC <> 0) Then
        hInst = App.hInstance
        m_hBmp = LoadImageByNum(hInst, 48, IMAGE_BITMAP, 0, 0, LR_LOADMAP3DCOLORS)
        If (m_hBmp <> 0) Then
            m_hBMPOLd = SelectObject(m_HDC, m_hBmp)
        End If
    End If
End Sub
Private Sub UserControl_Resize()
    UserControl.Width = 40 * Screen.TwipsPerPixelX
    UserControl.Height = 40 * Screen.TwipsPerPixelY
End Sub

Private Sub UserControl_Terminate()
    Debug.Print "Terminate"
    ' Remove any new menus we have created:
    Clear
    ' Destroy the sub class:
    pDestroySubClass
    ' Remove the graphics:
    If (m_HDC <> 0) Then
        If (m_hBmp <> 0) Then
            SelectObject m_HDC, m_hBMPOLd
            DeleteObject m_hBmp
        End If
        If (m_hFntOld <> 0) Then
            SelectObject m_HDC, m_hFntOld
        End If
        DeleteObject m_HDC
    End If
    ' Remove the image list worker DCs:
    ImageListClearUpWorkDCS
    ' Clear the non-client object, removing any fonts:
    Set m_cNCM = Nothing
End Sub
